Package: SVAlignR
Version: 0.9.2
Date: 2025-09-20
Title: Recovering Structure of Long Molecules from Structural Variation
        Data
Authors@R: person(given = c("Kevin", "R."), family = "Coombes",
                  role = c("aut", "cre"), email = "krc@silicovore.com")
Description: Implements a method to combine multiple levels of multiple
  sequence alignment to uncover the structure of complex DNA rearrangements.
Depends: R (>= 4.4)
Imports: methods, graphics, grDevices, oompaBase, Biostrings,
        NameNeedle, dendextend, ape, stringr, igraph, Polychrome,
        colorspace
Suggests: msa, viridisLite, R.rsp, knitr, rmarkdown
VignetteBuilder: R.rsp
License: Apache License (== 2.0)
URL: http://oompa.r-forge.r-project.org/
NeedsCompilation: no
Packaged: 2025-09-20 12:54:30 UTC; KRC
Author: Kevin R. Coombes [aut, cre]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: CRAN
Date/Publication: 2025-09-24 08:40:02 UTC
Built: R 4.6.0; ; 2026-04-28 21:54:44 UTC; unix
