Package: SVAlignR
Version: 0.10.4
Date: 2026-06-18
Title: Recovering Structure of Long Molecules from Structural Variation
        Data
Authors@R: person(given = c("Kevin", "R."), family = "Coombes",
                  role = c("aut", "cre"), email = "krc@silicovore.com")
Description: Implements a method to combine multiple levels of multiple
  sequence alignment to uncover the structure of complex DNA rearrangements.
Depends: R (>= 4.4)
Imports: methods, graphics, grDevices, oompaBase, Biostrings,
        NameNeedle, dendextend, ape, stringr, igraph, Polychrome,
        colorspace
Suggests: msa, viridisLite, R.rsp, knitr, rmarkdown
VignetteBuilder: R.rsp
License: Apache License (== 2.0)
URL: http://oompa.r-forge.r-project.org/
Author: Kevin R. Coombes [aut, cre]
Maintainer: Kevin R. Coombes <krc@silicovore.com>
Repository: R-Forge
Repository/R-Forge/Project: clonefinder
Repository/R-Forge/Revision: 195
Repository/R-Forge/DateTimeStamp: 2026-06-18 20:20:13
Date/Publication: 2026-06-18 20:20:13
NeedsCompilation: no
Packaged: 2026-06-22 10:44:09 UTC; rforge
