###
### NAMESPACE
###

import("methods")
importMethodsFrom("oompaBase", "plot", "image")

importFrom("graphics", "image", "mtext", "text", "abline", "lines")
importFrom("grDevices", "topo.colors")
importFrom("stats", "aggregate",
           "hclust", "as.dendrogram",
           "as.dist", "cutree", "heatmap")
importFrom("utils", "adist", "write.csv")

### Computational models
importFrom("NameNeedle", "needleScores", "defaultNeedleParams")
#importFrom("msa", "msa", "msaConsensusSequence", "msaClustalW")
importFrom("Biostrings", "AAStringSet")

### graphing
importFrom("dendextend", "set", "%>%")
importFrom("ape", "as.phylo")
importFrom("Polychrome", "createPalette", "dark.colors")
importFrom("igraph", "graph_from_edgelist", "graph_from_adjacency_matrix",
           "set_vertex_attr", "vertex_attr", "set_edge_attr", "edge_attr")
importFrom("stringr", "str_count", "str_locate")
importFrom("colorspace", "hex2RGB")

### Exports
exportClass("Cipher")
export("Cipher", "encode", "decode")

### SequenceClusters
exportClass("SequenceCluster")
export("SequenceCluster", "updateClusters", "heat")
export("makeSubsMatrix", "alignCluster", "alignAllClusters")

exportClass("AlignedCluster")

exportMethods("plot", "image")

### StringGraphs
exportClass("StringGraph")
export("MotifGraph", "DecompositionGraph", "exportSG")

## KMers
export("makeWords", "countWords", "plotWords")

### Breakpoints
exportClass("Breakpoints")
export("Breakpoints")

### De Bruijn Graphs
exportClass("DeBruijn")
export("deBruijn")

## Evidence for putative cycles
export("getEvidence")
